@@ -4,13 +4,13 @@ from __future__ import print_function, absolute_import
 
 import itertools
 from decimal import Decimal
-from copy import deepcopy, copy
+from copy import copy
 import numpy as np
 
 from .TreeNode import TreeNode
-from .TreeStyle import TreeStyle
+from .TreeStyle import TreeStyle, COLORS2
+from .StyleChecker import StyleChecker
 from .Coords import Coords
-from .Drawing import Drawing
 from .TreeParser import TreeParser, FastTreeParser
 from .TreeWriter import NewickWriter
 from .Treemod import TreeMod
@@ -18,13 +18,13 @@ from .PCM import PCM
 from .Rooter import Rooter
 from .NodeAssist import NodeAssist
 from .utils import ToytreeError, fuzzy_match_tipnames, normalize_values
-
+from .Render import ToytreeMark
+from .CanvasSetup import CanvasSetup
 
 """
 Test for speed improvements: 
 - reduce deepcopies
 - reduce traversals.
-
 """
 
 
@@ -51,22 +51,21 @@ class ToyTree(object):
     ----------
     ...
     """
-    def __init__(self, newick=None, tree_format=0, fixed_order=None):
+    def __init__(self, newick=None, tree_format=0, **kwargs):
 
         # if loading from a Toytree then inherit that trees draw style
         inherit_style = False
 
-        # load from a TreeNode
+        # load from a TreeNode and detach. Must have .idx attributes on nodes.
         if isinstance(newick, TreeNode):
-            # detach this node from any parents so it becomes root
-            self.treenode = deepcopy(newick).detach()
+            self.treenode = newick.detach()
 
-        # load TreeNode from a ToyTree
+        # load TreeNode from a ToyTree (user should just use .copy())
         elif isinstance(newick, ToyTree):
             self.treenode = newick.treenode
             inherit_style = True
 
-        # prase a str, URL, or file
+        # parse a str, URL, or file
         elif isinstance(newick, (str, bytes)):
             self.treenode = TreeParser(newick, tree_format).treenodes[0]
 
@@ -80,16 +79,16 @@ class ToyTree(object):
         self.idx_dict = {}
 
         # set tips order if fixing for multi-tree plotting (default None)
-        self._fixed_order = None
-        self._fixed_idx = list(range(self.ntips))
-        if fixed_order:
-            if not isinstance(fixed_order, (list, tuple)):
-                raise ToytreeError("fixed_order arg should be a list")
-            self._set_fixed_order(fixed_order)
+        # self._fixed_order = None
+        # self._fixed_idx = list(range(self.ntips))
+        # if fixed_order:
+            # if not isinstance(fixed_order, (list, tuple)):
+                # raise ToytreeError("fixed_order arg should be a list")
+            # self._set_fixed_order(fixed_order)
 
         # ladderize the tree unless user fixed order and wants it not.
-        if not self._fixed_order:
-            self.treenode.ladderize()
+        # if not self._fixed_order:
+        self.treenode.ladderize()
 
         # Object for storing default plot settings or saved styles.
         # Calls several update functions when self.draw() to fit canvas.
@@ -101,6 +100,7 @@ class ToyTree(object):
         # Object for plot coordinates. Calls .update() whenever tree modified.
         self._coords = Coords(self)
         self._coords.update()
+        # if not kwargs.get("copy"):
 
         # Object for modifying trees beyond root, prune, drop
         self.mod = TreeMod(self)
@@ -119,18 +119,18 @@ class ToyTree(object):
         return len(self.treenode)
 
 
-    def _set_fixed_order(self, fixed_order):
-        """
-        Setting fixed_idx is important for when nodes are rotated, and edges
-        are different lengths, b/c it allows updating coords to match up.
-        """
-        if fixed_order:
-            if set(fixed_order) != set(self.treenode.get_leaf_names()):
-                raise ToytreeError(
-                    "fixed_order must include same tipnames as tree")
-            self._fixed_order = fixed_order
-            names = self.treenode.get_leaf_names()[::-1]
-            self._fixed_idx = [names.index(i) for i in self._fixed_order]
+    # def _set_fixed_order(self, fixed_order):
+    #     """
+    #     Setting fixed_idx is important for when nodes are rotated, and edges
+    #     are different lengths, b/c it allows updating coords to match up.
+    #     """
+    #     if fixed_order:
+    #         if set(fixed_order) != set(self.treenode.get_leaf_names()):
+    #             raise ToytreeError(
+    #                 "fixed_order must include same tipnames as tree")
+    #         self._fixed_order = fixed_order
+    #         names = self.treenode.get_leaf_names()[::-1]
+    #         self._fixed_idx = [names.index(i) for i in self._fixed_order]
 
     # --------------------------------------------------------------------
     # properties are not changeable by the user
@@ -238,19 +238,19 @@ class ToyTree(object):
             widths. In the range(2, 12) typically.
         """
         elist = []
-        for cidx in self._coords.edges[:, 1]:
-            node = self.treenode.search_nodes(idx=cidx)[0]
+        for eidx in self._coords.edges[:, 1]:
+            node = self.idx_dict[eidx]
             elist.append(
-                # (node.__getattribute__(feature) if hasattr(node, feature) else "")
                 (getattr(node, feature) if hasattr(node, feature) else "")
-                )
+            )
         elist = np.array(elist)
         if normalize:
             elist = normalize_values(elist)
         return elist
 
 
-    def get_edge_values_from_dict(self, node_value_dict=None, include_stem=True):
+
+    def get_edge_values_mapped(self, node_mapping=None, include_stem=True):
         """
         Enter a dictionary mapping node 'idx' or tuple of tipnames to values 
         that you want mapped to the stem and descendant edges that node. 
@@ -270,45 +270,67 @@ class ToyTree(object):
 
         Example 1: 
           tre = toytree.tree("((a,b),(c,d));")
-          tre.get_edge_values_from_dict({5: 'green', 6: 'red'})
+          tre.get_edge_values_mapped({5: 'green', 6: 'red'})
           # ['green', 'green', 'green', 'red', 'red', 'red']
 
         Example 2: 
           tre = toytree.tree("((a,b),(c,d));")
-          tre.get_edge_values_from_dict({(a, b): 'green', (c, d): 'red'})          
+          tre.get_edge_values_mapped({(a, b): 'green', (c, d): 'red'})          
           # ['green', 'green', 'green', 'red', 'red', 'red']
+
+        Example 3:
+          tre = toytree.tree("((a,b),(c,d));")
+          tre.get_edge_values_mapped({10, 13})
+          # ['green', 'green', 'green', 'red', 'red', 'red']
+
         """
-        # map node idxs to the order in which edges are plotted
-        idxs = {j: i for (i, j) in enumerate(self.get_edge_values())}
         values = [None] * self._coords.edges.shape[0]
-        if node_value_dict is None:
+        if node_mapping is None:
             return values
 
-        # convert tipname lists to node idxs
-        rmap = {}
-        for (key, val) in node_value_dict.items():
-            if isinstance(key, (str, tuple)):
-                node = fuzzy_match_tipnames(self, key, None, None, True, False)
-                rmap[node.idx] = val
-            else:
-                rmap[key] = val
-        node_value_dict = rmap
+        if isinstance(node_mapping, set):
+            cols = iter(COLORS2)
+            node_mapping = {i: next(cols) for i in node_mapping}
 
-        # map over tree
-        for node in self.treenode.traverse("levelorder"):
-            if node.idx in node_value_dict:
+        # build ...
+        rmap = {}
+        for key in node_mapping:
+
+            # if it is a node idx
+            if isinstance(key, int):
+                rmap[key] = node_mapping[key]
+            else:
+                ns = NodeAssist(self, key, None, None)
+                kidx = ns.get_mrca().idx
+                rmap[kidx] = node_mapping[key]
+
+        # ....
+        for idx in self.idx_dict:
+            node = self.idx_dict[idx]
+            if idx in rmap:
 
                 # add value to stem edge
                 if include_stem:
                     if not node.is_root():
-                        values[idxs[node.idx]] = node_value_dict[node.idx]
+                        values[idx] = rmap[idx]
 
                 # add value to descendants edges
                 for desc in node.get_descendants():
-                    values[idxs[desc.idx]] = node_value_dict[node.idx]
+                    values[desc.idx] = rmap[idx]
         return values
 
 
+
+    def get_edge_values_from_dict(self, node_value_dict=None, include_stem=True):
+        """
+        No longer supported. See get_edge_values_mapped()
+        """
+        print("Warning: get_edge_values_from_dict no longer supported."
+              " See get_edge_values_mapped() as a replacement.")
+        return self.get_edge_values_mapped(node_value_dict, include_stem)
+
+
+
     def get_mrca_idx_from_tip_labels(self, names=None, wildcard=None, regex=None):
         """
         Returns the node idx label of the most recent common ancestor node 
@@ -331,7 +353,7 @@ class ToyTree(object):
         return [idx] + [i.idx for i in node.get_descendants()]
 
 
-    def get_node_coordinates(self, layout=None):
+    def get_node_coordinates(self, layout=None, use_edge_lengths=True):
         """
         Returns coordinate locations of nodes in the tree as an array. Each
         row is an (x, y) coordinate, ordered by the 'idx' feature of nodes.
@@ -339,7 +361,12 @@ class ToyTree(object):
         returned using .get_tip_coordinates().
         """
         # if layout argument then set style and update coords.
-        return self._coords.verts
+        if layout is None:
+            layout = self.style.layout
+        if layout == 'c':
+            return self._coords.get_radial_coords(use_edge_lengths)
+        else:
+            return self._coords.get_linear_coords(layout, use_edge_lengths)
 
 
     def get_node_values(
@@ -460,21 +487,24 @@ class ToyTree(object):
                 return {i.idx: i.name for i in nodes}
 
 
-    def get_tip_coordinates(self, axis=None, layout=None):
+    def get_tip_coordinates(self, layout=None, use_edge_lengths=True):
         """
         Returns coordinates of the tip positions for a tree. If no argument
         for axis then a 2-d array is returned. The first column is the x 
         coordinates the second column is the y-coordinates. If you enter an 
         argument for axis then a 1-d array will be returned of just that axis.
         """
-        # TODO: if layout then update coords for new layout (e.g., 'c')
+
+        # one could imagine a very simple method like this, but to accomodate
+        # circ and unrooted layout we'll want a better option.
+        # return np.arange(self.ntips) + self.style.xbaseline + ybase...
+
+        # if no layout provided then use current style
+        if layout is None:
+            layout = self.style.layout
 
         # get coordinates array
-        coords = self.get_node_coordinates()
-        if axis == 'x':
-            return coords[:self.ntips, 0]
-        elif axis == 'y':
-            return coords[:self.ntips, 1]
+        coords = self.get_node_coordinates(layout, use_edge_lengths)
         return coords[:self.ntips]
 
 
@@ -492,18 +522,19 @@ class ToyTree(object):
         Parameters:
             idx (int): index label of a node.
         """
-        if idx:
-            treenode = self.treenode.search_nodes(idx=idx)[0]
-            if self._fixed_order:
-                return [i for i in self._fixed_order if i in 
-                        treenode.get_leaf_names()]
-            else:
-                return treenode.get_leaf_names()[::-1]                
+        if idx is not None:
+            treenode = self.idx_dict[idx]
+            # if self._fixed_order:
+                # return [str(i) for i in self._fixed_order if i in 
+                        # treenode.get_leaf_names()]
+            # else:
+            return [str(i) for i in treenode.get_leaf_names()[::-1]]
+
         else:
-            if self._fixed_order:
-                return self._fixed_order
-            else:
-                return self.treenode.get_leaf_names()[::-1]
+            # if self._fixed_order:
+                # return [str(i) for i in self._fixed_order]
+            # else:
+            return [str(i) for i in self.treenode.get_leaf_names()[::-1]]
 
 
     def set_node_values(self, feature, values=None, default=None):
@@ -546,15 +577,15 @@ class ToyTree(object):
         nself = self.copy()
 
         # make default ndict using idxs, regardless of values
-        ndict = nself.get_feature_dict("idx", None)
+        ndict = nself.idx_dict
 
-        # if numeric keys in values then use idx, else use names.
+        # if first value is a string then use name_dict instead of idx_dict
         if values:
-            if isinstance(list(values.keys())[0], (int, float)):
-                # ndict = nself.get_feature_dict("idx", None)
+            val0 = list(values.keys())[0]
+            if isinstance(val0, (str, bytes)):
+                ndict = {ndict[i].name: ndict[i] for i in ndict}
+            elif isinstance(val0, int):
                 pass
-            elif isinstance(list(values.keys())[0], (str, bytes)):
-                ndict = nself.get_feature_dict("name", None)
             else:
                 raise ToytreeError("dictionary keys should be int or str")
 
@@ -562,9 +593,9 @@ class ToyTree(object):
         if feature == "idx":
             raise ToytreeError("cannot modify idx values.")
         if feature == "height":
-            raise ToytreeError("modifying heights not yet supported, coming..")
+            raise ToytreeError("modifying heights not supported, use dist.")
 
-        # set everyone to a default value
+        # set everyone to a default value for this attribute
         if default is not None:
             for key in ndict:
                 node = ndict[key]
@@ -596,9 +627,29 @@ class ToyTree(object):
         return nself
 
 
+
     def copy(self):
-        """ returns a deepcopy of the tree object"""
-        return deepcopy(self)
+        """ Returns a new ToyTree equivalent to a deepcopy (but faster) """
+
+        # copy treenodes w/ topology, node attrs, nnodes, ntips, and idx_dict
+        nself = ToyTree(
+            self.treenode._clone(), 
+            # fixed_order=self._fixed_order,
+            copy=True,
+        )
+
+        # update style dicts
+        nself.style = self.style.copy()
+
+        # update coords by copying instead of coords.update
+        # nself._coords.edges = nself._coords.get_edges()
+        # nself._coords.verts = self._coords.verts.copy()
+        return nself
+
+
+    # def copy(self):
+    #     """ returns a deepcopy of the tree object"""
+    #     return deepcopy(self)
 
 
     def is_rooted(self):
@@ -635,7 +686,7 @@ class ToyTree(object):
         """
         nself = self.copy()
         nself.treenode.ladderize(direction=direction)
-        nself._fixed_order = None
+        # nself._fixed_order = None
         nself._coords.update()
         return nself
 
@@ -748,7 +799,7 @@ class ToyTree(object):
         neworder = [revd[i] for i in range(self.ntips)]
 
         # returns a new tree (i.e., copy) modified w/ a fixed order
-        nself = ToyTree(self.newick, fixed_order=neworder)
+        nself = ToyTree(self.newick)  #, fixed_order=neworder)
         nself._coords.update()
         return nself
 
@@ -772,49 +823,49 @@ class ToyTree(object):
 
 
 
-    def speciate(self, idx, name=None, dist_prop=0.5):
-        """
-        Split an edge to create a new tip in the tree as in a speciation event.
-        """
-        # make a copy of the toytree
-        nself = self.copy()
+    # def speciate(self, idx, name=None, dist_prop=0.5):
+    #     """
+    #     Split an edge to create a new tip in the tree as in a speciation event.
+    #     """
+    #     # make a copy of the toytree
+    #     nself = self.copy()
 
-        # get Treenodes of selected node and parent 
-        ndict = nself.get_feature_dict('idx')
-        node = ndict[idx]
-        parent = node.up
+    #     # get Treenodes of selected node and parent 
+    #     ndict = nself.get_feature_dict('idx')
+    #     node = ndict[idx]
+    #     parent = node.up
 
-        # get new node species name
-        if not name:
-            if node.is_leaf():
-                name = node.name + ".sis"
-            else:
-                names = nself.get_tip_labels(idx=idx)
-                name = "{}.sis".format("_".join(names))
+    #     # get new node species name
+    #     if not name:
+    #         if node.is_leaf():
+    #             name = node.name + ".sis"
+    #         else:
+    #             names = nself.get_tip_labels(idx=idx)
+    #             name = "{}.sis".format("_".join(names))
 
-        # create new speciation node between them at dist_prop dist.
-        newnode = parent.add_child(
-            name=parent.name + ".spp",
-            dist=node.dist * dist_prop
-        )
+    #     # create new speciation node between them at dist_prop dist.
+    #     newnode = parent.add_child(
+    #         name=parent.name + ".spp",
+    #         dist=node.dist * dist_prop
+    #     )
 
-        # connect original node to speciation node.
-        node.up = newnode
-        node.dist = node.dist - newnode.dist
-        newnode.add_child(node)
+    #     # connect original node to speciation node.
+    #     node.up = newnode
+    #     node.dist = node.dist - newnode.dist
+    #     newnode.add_child(node)
 
-        # drop original node from original parent child list
-        parent.children.remove(node)
+    #     # drop original node from original parent child list
+    #     parent.children.remove(node)
 
-        # add new tip node (new sister) and set same dist as onode
-        newnode.add_child(
-            name=name,
-            dist=node.up.height,
-        )
+    #     # add new tip node (new sister) and set same dist as onode
+    #     newnode.add_child(
+    #         name=name,
+    #         dist=node.up.height,
+    #     )
 
-        # update toytree coordinates
-        nself._coords.update()
-        return nself        
+    #     # update toytree coordinates
+    #     nself._coords.update()
+    #     return nself        
 
 
 
@@ -907,7 +958,7 @@ class ToyTree(object):
         tree_style=None,
         height=None,
         width=None,
-        axes=None,        
+        axes=None,    
         layout=None,
         tip_labels=None,
         tip_labels_colors=None,
@@ -931,13 +982,16 @@ class ToyTree(object):
         xbaseline=None,
         ybaseline=None,
         admixture_edges=None,
+        shrink=None,
+        fixed_order=None,
+        fixed_position=None,
         **kwargs):
         """
         Plot a Toytree tree, returns a tuple of Toyplot (Canvas, Axes) objects.
 
         Parameters:
         -----------
-        tree_style: str
+        tree_style (or ts): str
             One of several preset styles for tree plotting. The default is 'n'
             (normal). Other options inlude 'c' (coalescent), 'd' (dark), and
             'm' (multitree). You also create your own TreeStyle objects.
@@ -1013,15 +1067,6 @@ class ToyTree(object):
             height, width, color). Example: [(4, 3, 50000, 3, 'red')]
 
         """
-        # allow ts as a shorthand for tree_style
-        if kwargs.get("ts"):
-            tree_style = kwargs.get("ts")
-
-        # pass a copy of this tree so that any mods to .style are not saved
-        nself = deepcopy(self)
-        if tree_style:
-            nself.style.update(TreeStyle(tree_style[0]))
-
         # update kwargs to merge it with user-entered arguments:
         userargs = {
             "height": height,
@@ -1049,31 +1094,70 @@ class ToyTree(object):
             "xbaseline": xbaseline, 
             "ybaseline": ybaseline,
             "admixture_edges": admixture_edges,
-            # "orient": orient,
+            "shrink": shrink,
+            "fixed_order": fixed_order,
+            "fixed_position": fixed_position
         }
 
-        # update kwargs with userargs, update style w/ kwargs except empty ones
-        kwargs.update(userargs)
-        censored = {i: j for (i, j) in kwargs.items() if j is not None}
-        nself.style.update(censored)
+        # shortcut name for tree style
+        if kwargs.get("ts"):
+            tree_style = kwargs.get("ts")
 
-        # warn user if they entered kwargs that weren't recognized:
+        # use a base style preset over which other options override
+        if tree_style:
+            curstyle = TreeStyle(tree_style[0])
+
+        # or use current tree settings (DEFAULT unless changed by user)
+        else:           
+            curstyle = self.style.copy()
+
+        # optionally override current style with style args entered to draw()
+        kwargs.update(userargs)
+        user = dict([
+            ("_" + i, j) if isinstance(j, dict) else (i, j)
+            for (i, j) in kwargs.items() if j is not None
+        ])
+        curstyle.update(user)
+
+        # warn user if they entered kwargs that arent't supported:
         allkeys = list(userargs.keys()) + ["debug", "ts"]
         unrecognized = [i for i in kwargs if i not in allkeys]
         if unrecognized:
-            print("unrecognized arguments skipped: {}".format(unrecognized))
-            print("check the docs, argument names may have changed.")
+            print("unrecognized arguments skipped: {}"
+                  "\ncheck the docs, argument names may have changed."
+                  .format(unrecognized))
 
-        # Init Drawing class object.
-        draw = Drawing(nself)
+        # update coords based on layout
+        edges = self._coords.get_edges()
+        if layout == 'c':
+            verts = self._coords.get_radial_coords(curstyle.use_edge_lengths)
+        else:
+            verts = self._coords.get_linear_coords(
+                curstyle.layout, 
+                curstyle.use_edge_lengths,
+                fixed_order,
+                fixed_position,
+                )
 
-        # Debug returns the object to test with.
-        if kwargs.get("debug"):
-            return draw
+        # check all styles
+        fstyle = StyleChecker(self, curstyle).style
 
-        # Make plot. If user provided explicit axes then include them.
-        canvas, axes = draw.update(axes=axes)
-        return canvas, axes
+        # debugging returns the mark and prints the modified kwargs
+        if kwargs.get('debug'):
+            print(user)
+            return fstyle
+
+        # get canvas and axes
+        cs = CanvasSetup(self, axes, fstyle)
+        canvas = cs.canvas
+        axes = cs.axes
+
+        # generate toyplot Mark
+        mark = ToytreeMark(ntable=verts, etable=edges, **fstyle.to_dict())
+
+        # add mark to axes
+        axes.add_mark(mark)
+        return canvas, axes, mark
 
 
 
